# settings.py
# Anzeige der Einstellungen des LoRa-Moduls

#####################################################################################
#                            Import und Instanziierungen                            #
#####################################################################################

import sys
from time import sleep
from machine import UART
uart1 = UART(1, baudrate = 57600, tx = 12, rx = 13)

from lora import send_command, read_response, response_to_list, lora_init, send_message

####################################################################################
#                 Anzeige aller Einstellungen des LORA-Modul                       #
####################################################################################


lora_address = lora_init(uart1) # Rückgabewert ist '???', wenn keinen Antwort vom LoRa-Modul 
if lora_address == '???':
    print('Keine Verbindung zum LORA-Modul!')
    print('RESET-Knopf am TTGO betätigen!')
    sys.exit() # Programm beenden, wenn keine Antwort vom LoRa-Modul
    
print('Einstellungen des angeschlossenen LoRa-Moduls')
print()


# Adresse
send_command(uart1, 'AT+ADDRESS?')
print('Adresse: ', read_response(uart1))
print()

# Netzwerk-ID
send_command(uart1, 'AT+NETWORKID?')
print('NETWORKID (Default: 18): ', read_response(uart1))
print()

# Netzwerk-ID
send_command(uart1, 'AT+BAND?')
print('Frequenz (868500000): ', read_response(uart1))
print()

# Parameter
send_command(uart1, 'AT+PARAMETER?')
print('Parameter (Default: 9,7,1,12): ', read_response(uart1))
print()

# Baudrate
send_command(uart1, 'AT+IPR?')
print('Baudrate (Default: 57600): ', read_response(uart1))
print()

# Sendeleistung
send_command(uart1, 'AT+CRFOP?')
print('Sendeleistung (Default; Max in EU: 14): ', read_response(uart1))
print()

# UID (Unique Identifyer)
send_command(uart1, 'AT+UID?')
print('UID: ', read_response(uart1))
print()

# Version (Unique Identifyer)
send_command(uart1, 'AT+VER?')
print('Version: ', read_response(uart1))
print()

# Passwort (CPIN)
send_command(uart1, 'AT+CPIN?')
print('Passwort (CPIN): ', read_response(uart1))
print()



